@@ -21,8 +21,6 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 import os
 import shutil
 import tempfile
@@ -45,7 +43,7 @@ from ...errors import (
     NoWorkingTree,
     )
 from ...option import Option
-from ...trace import mutter, note
+from ...trace import mutter, note, warning
 from ...transport import get_transport
 from ...workingtree import WorkingTree
 
@@ -62,7 +60,6 @@ from .config import (
     )
 from .util import (
     debuild_config,
-    get_build_architecture,
     )
 
 dont_purge_opt = Option(
@@ -294,9 +291,9 @@ def _get_distiller(
             tree, subpath, upstream_provider, top_level=top_level,
             use_existing=use_existing)
     elif build_type == BUILD_TYPE_NATIVE:
-        return NativeSourceDistiller(tree, subpath)
+        return NativeSourceDistiller(tree, subpath, use_existing=use_existing)
     else:
-        return FullSourceDistiller(tree, subpath, upstream_provider)
+        return FullSourceDistiller(tree, subpath, upstream_provider, use_existing=use_existing)
 
 
 class cmd_builddeb(Command):
@@ -481,6 +478,7 @@ class cmd_builddeb(Command):
             quick=False, reuse=False, native=None,
             source=False, revision=None, package_merge=None,
             strict=False, guess_upstream_branch_url=False):
+        from debian.changelog import ChangelogParseError
         from .builder import DebBuild
         from .config import UpstreamMetadataSyntaxError
         from .hooks import run_hook
@@ -491,7 +489,7 @@ class cmd_builddeb(Command):
             find_changelog,
             find_previous_upload,
             tree_contains_upstream_source,
-            changes_filename,
+            find_changes_files,
             )
 
         location, build_options, source = self._branch_and_build_options(
@@ -516,8 +514,13 @@ class cmd_builddeb(Command):
 
             contains_upstream_source = tree_contains_upstream_source(
                     tree, subpath)
-            (changelog, top_level) = find_changelog(
-                tree, subpath, merge=not contains_upstream_source)
+            try:
+                (changelog, top_level) = find_changelog(
+                    tree, subpath, merge=not contains_upstream_source)
+            except ChangelogParseError as e:
+                raise BzrCommandError(
+                    gettext("Unable to parse changelog: %s") %
+                    e)
 
             if package_merge:
                 try:
@@ -568,28 +571,26 @@ class cmd_builddeb(Command):
                 run_hook(tree, 'post-build', config, wd=build_source_dir)
                 if not dont_purge:
                     builder.clean()
-                if source:
-                    arch = "source"
-                else:
-                    arch = get_build_architecture()
-                changes = changes_filename(
-                    changelog.package, changelog.version, arch)
-                changes_path = os.path.join(build_dir, changes)
-                if not os.path.exists(changes_path):
+                changes_paths = []
+                for kind, entry in find_changes_files(
+                        build_dir, changelog.package, changelog.version):
+                    changes_paths.append(entry.path)
+                if not changes_paths:
                     if result_dir is not None:
                         raise BzrCommandError(
                             "Could not find the .changes "
-                            "file from the build: %s" % changes_path)
+                            "file from the build: %s" % build_dir)
+                    return
+                if is_local:
+                    target_dir = result_dir or default_result_dir
+                    target_dir = os.path.join(
+                            urlutils.local_path_from_url(location),
+                            target_dir)
                 else:
-                    if is_local:
-                        target_dir = result_dir or default_result_dir
-                        target_dir = os.path.join(
-                                urlutils.local_path_from_url(location),
-                                target_dir)
-                    else:
-                        target_dir = "."
-                    if not os.path.exists(target_dir):
-                        os.makedirs(target_dir)
+                    target_dir = "."
+                if not os.path.exists(target_dir):
+                    os.makedirs(target_dir)
+                for changes_path in changes_paths:
                     dget_changes(changes_path, target_dir)
 
 
@@ -724,10 +725,6 @@ class cmd_merge_upstream(Command):
         'snapshot', help="Merge a snapshot from the "
         "upstream branch rather than a new upstream release.")
 
-    launchpad_opt = Option(
-        'launchpad',
-        help='Use Launchpad to find upstream locations.')
-
     force_pristine_tar_opt = Option(
         'force-pristine-tar', help=(
             'Force the use of pristine-tar, even if no '
@@ -738,27 +735,17 @@ class cmd_merge_upstream(Command):
         help='Command to run for creating an upstream tarball from a '
         'VCS snapshot.')
 
+    guess_upstream_branch_url_opt = Option(
+        'guess-upstream-branch-url', help=(
+            'Guess upstream branch URL if unknown '
+            '(requires upstream-ontologist)'))
+
     takes_options = [
         package_opt, version_opt,
         distribution_opt, directory_opt, last_version_opt,
         force_opt, 'revision', 'merge-type',
-        snapshot_opt, launchpad_opt, force_pristine_tar_opt,
-        dist_command_opt]
-
-    def _add_changelog_entry(self, tree, subpath, package, version,
-                             distribution_name, changelog):
-        from .merge_upstream import (
-            DchError,
-            changelog_add_new_version)
-        try:
-            changelog_add_new_version(
-                tree, subpath, version, distribution_name, changelog, package)
-        except DchError as e:
-            note(e)
-            raise BzrCommandError(
-                    'Adding a new changelog stanza after the '
-                    'merge had completed failed. Add the new changelog '
-                    'entry yourself, review the merge, and then commit.')
+        snapshot_opt, force_pristine_tar_opt,
+        dist_command_opt, guess_upstream_branch_url_opt]
 
     def run(self, location: Optional[str] = None,
             upstream_branch: Optional[str] = None,
@@ -766,13 +753,15 @@ class cmd_merge_upstream(Command):
             distribution: Optional[str] = None, package: Optional[str] = None,
             directory: str = ".", revision=None, merge_type=None,
             last_version: Optional[str] = None,
-            force: Optional[bool] = None, snapshot: bool = False,
-            launchpad: bool = False, force_pristine_tar: bool = False,
-            dist_command: Optional[str] = None):
+            force: Optional[bool] = None, snapshot: Optional[bool] = None,
+            force_pristine_tar: bool = False,
+            dist_command: Optional[str] = None,
+            guess_upstream_branch_url: bool = False):
         from debian.changelog import Version
 
         from .hooks import run_hook
         from .merge_upstream import (
+            changelog_add_new_version,
             do_merge,
             get_tarballs,
             PreviousVersionTagMissing,
@@ -790,6 +779,7 @@ class cmd_merge_upstream(Command):
             NoWatchFile,
             )
         from .util import (
+            detect_version_kind,
             guess_build_type,
             get_files_excluded,
             tree_contains_upstream_source,
@@ -809,6 +799,19 @@ class cmd_merge_upstream(Command):
                     "package name, which is needed to know the name to "
                     "give the .orig.tar.gz. Please specify --package.")
 
+            if snapshot is None:
+                if current_version is not None:
+                    version_kind = detect_version_kind(current_version)
+                else:
+                    version_kind = None
+            elif snapshot is True:
+                version_kind = "snapshot"
+            elif snapshot is False:
+                version_kind = "release"
+
+            if version_kind is None:
+                version_kind = "release"
+
             try:
                 files_excluded = get_files_excluded(tree, subpath, top_level)
             except NoSuchFile as e:
@@ -829,14 +832,6 @@ class cmd_merge_upstream(Command):
                 raise BzrCommandError(gettext(
                     "Merge upstream in native mode is not supported."))
 
-            if launchpad:
-                from .launchpad import (
-                    get_upstream_branch_url as lp_get_upstream_branch_url,
-                    )
-                upstream_branch = lp_get_upstream_branch_url(
-                    package, distribution_name, distribution)
-                note(gettext("Using upstream branch %s") % upstream_branch)
-
             if upstream_branch is not None:
                 upstream_branch = Branch.open(upstream_branch)
             elif location is not None and not os.path.isfile(location):
@@ -849,8 +844,27 @@ class cmd_merge_upstream(Command):
                 note(gettext("Using upstream branch %s (from configuration)"),
                      config.upstream_branch)
                 upstream_branch = Branch.open(config.upstream_branch)
-            else:
-                upstream_branch = None
+            elif upstream_branch is None and guess_upstream_branch_url:
+                try:
+                    from upstream_ontologist.guess import (
+                        guess_upstream_metadata,
+                        )
+                except ModuleNotFoundError:
+                    warning(
+                        'upstream-ontologist not available, '
+                        'not guess upstream branch URL')
+                    upstream_branch = None
+                else:
+                    guessed_upstream_metadata = guess_upstream_metadata(
+                        tree.abspath(subpath), trust_package=False,
+                        net_access=True, consult_external_directory=False)
+                    guessed_repo = guessed_upstream_metadata.get('Repository')
+                    if guessed_repo:
+                        note('Opening upstream-ontologist provided branch %s',
+                             guessed_repo)
+                        upstream_branch = Branch.open(guessed_repo)
+                    else:
+                        upstream_branch = None
 
             if dist_command:
                 def create_dist(tree, package, version, target_dir):
@@ -862,7 +876,7 @@ class cmd_merge_upstream(Command):
             if upstream_branch is not None:
                 upstream_branch_source = UpstreamBranchSource.from_branch(
                     upstream_branch, config=config, local_dir=tree.controldir,
-                    create_dist=create_dist, snapshot=snapshot)
+                    create_dist=create_dist, version_kind=version_kind)
             else:
                 upstream_branch_source = None
 
@@ -871,7 +885,7 @@ class cmd_merge_upstream(Command):
                     primary_upstream_source = UpstreamBranchSource.from_branch(
                         Branch.open(location), config=config,
                         local_dir=tree.controldir,
-                        create_dist=create_dist, snapshot=snapshot)
+                        create_dist=create_dist, version_kind=version_kind)
                 except NotBranchError:
                     primary_upstream_source = TarfileSource(location, version)
             else:
@@ -907,10 +921,10 @@ class cmd_merge_upstream(Command):
 
             if version is None and upstream_revisions is not None:
                 # Look up the version from the upstream revision
-                version = upstream_branch_source.get_version(
+                unmangled_version, version = upstream_branch_source.get_version(
                     package, current_version, upstream_revisions[None])
             elif version is None and primary_upstream_source is not None:
-                version = primary_upstream_source.get_latest_version(
+                unmangled_version, version = primary_upstream_source.get_latest_version(
                     package, current_version)
             if version is None:
                 if upstream_branch_source is not None:
@@ -923,7 +937,7 @@ class cmd_merge_upstream(Command):
                     raise BzrCommandError(gettext(
                         "You must specify the "
                         "version number using --version."))
-            note(gettext("Using version string %s.") % (version))
+            note(gettext("Using version string %s."), version)
             # Look up the revision id from the version string
             if (upstream_revisions is None and
                     upstream_branch_source is not None):
@@ -973,8 +987,8 @@ class cmd_merge_upstream(Command):
                 raise BzrCommandError(
                     gettext("Upstream version %s has already been merged.") %
                     version)
-            self._add_changelog_entry(
-                tree, subpath, package, version, distribution_name, changelog)
+            changelog_add_new_version(
+                tree, subpath, version, distribution_name, changelog, package)
             run_hook(tree, 'merge-upstream', config)
         if not need_upstream_tarball:
             note(gettext("An entry for the new upstream version has been "
@@ -1593,6 +1607,9 @@ def _build_helper(
                     local_tree, config, builder, target_dir)
 
 
+DEFAULT_BUILDER = "sbuild --source --source-only-changes --no-clean-source"
+
+
 class cmd_debrelease(Command):
     """Release a new version of a package.
 
@@ -1613,7 +1630,7 @@ class cmd_debrelease(Command):
         Option('skip-upload', help='Skip upload.'), builder_opt]
 
     def run(self, location='.', strict=True, skip_upload=False,
-            builder="sbuild --source --source-only-changes"):
+            builder=DEFAULT_BUILDER):
         from .release import release
         from .util import (
             dput_changes,
@@ -1628,10 +1645,10 @@ class cmd_debrelease(Command):
             release(local_tree, subpath)
 
             with tempfile.TemporaryDirectory() as td:
-                changes_file = _build_helper(
+                changes_files = _build_helper(
                         local_tree, subpath, local_tree.branch,
                         target_dir=(td if not skip_upload else None),
                         builder=builder)
                 if not skip_upload:
-                    dput_changes(changes_file)
+                    dput_changes(changes_files['source'])
             local_tree.branch.push(branch)
