@@ -18,8 +18,6 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 import re
 import sys
 
@@ -31,7 +29,7 @@ from ...trace import note, warning
 from debian.deb822 import Deb822
 from debian.changelog import Version
 
-from debmutate.vcs import source_package_vcs
+from debmutate.vcs import source_package_vcs, split_vcs_url
 
 
 def fixup_broken_git_url(url):
@@ -79,24 +77,9 @@ def fixup_broken_git_url(url):
 
 def vcs_git_url_to_bzr_url(url):
     """Convert a Vcs-Git string to a Breezy URL."""
-
-    # some packages seem to use [PATH] behind the URL to
-    # indicate a subdirectory inside of the versioned tree.
-    # (this is not documented in policy)
-
-    m = re.finditer(r' \[([^] ]+)\]', url)
-    try:
-        m = next(m)
-        url = url[:m.start()] + url[m.end():]
-        subpath = m.group(1)
-    except StopIteration:
-        subpath = None
+    (url, branch, subpath) = split_vcs_url(url)
 
     from breezy.git.urls import git_url_to_bzr_url
-    if ' -b ' in url:
-        (url, branch) = url.split(' -b ', 1)
-    else:
-        branch = None
 
     url = fixup_broken_git_url(url)
     url = git_url_to_bzr_url(url)
@@ -139,15 +122,14 @@ def vcs_cvs_url_to_bzr_url(location):
 
 
 def vcs_hg_url_to_bzr_url(url):
-    if ' -b ' in url:
-        (url, branch) = url.split(' -b ', 1)
-    else:
-        branch = None
+    (url, branch, subpath) = split_vcs_url(url)
 
     if branch:
         branch = urlutils.quote(branch, '')
         url = urlutils.join_segment_parameters(
             url, {'branch': branch})
+    if subpath:
+        url = urlutils.join(url, subpath)
     return url
 
 
@@ -177,7 +159,7 @@ def source_package_vcs_url(control):
 class AptDirectory(object):
     """Simple Bazaar directory service which uses dpkg Vcs-* fields."""
 
-    def look_up(self, name, url):
+    def look_up(self, name, url, purpose=None):
         if "/" in name:
             (name, version) = name.split("/", 1)
         else:
@@ -223,7 +205,7 @@ class AptDirectory(object):
 class DgitDirectory(object):
     """Directory that looks up the URL according to a Dgit control field."""
 
-    def look_up(self, name, url):
+    def look_up(self, name, url, purpose=None):
         if "/" in name:
             (name, version) = name.split("/", 1)
         else:
@@ -275,7 +257,7 @@ class DgitDirectory(object):
 class VcsDirectory(object):
     """Use local Vcs Directory."""
 
-    def look_up(self, name, url):
+    def look_up(self, name, url, purpose=None):
         from debian.deb822 import Deb822
         with open('debian/control', 'r') as f:
             source = Deb822(f)
@@ -287,5 +269,5 @@ def upstream_branch_alias(b):
     from .util import debuild_config
     with b.lock_read():
         tree = b.basis_tree()
-        config = debuild_config(tree)
+        config = debuild_config(tree, subpath='.')
         return directories.dereference(config.upstream_branch)
