@@ -25,8 +25,6 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 import calendar
 from contextlib import contextmanager, ExitStack
 import os
@@ -75,6 +73,9 @@ from .util import (
 from .upstream import (
     PackageVersionNotPresent,
     )
+from .upstream.branch import (
+    PreviousVersionTagMissing,
+    )
 from .upstream.pristinetar import (
     get_pristine_tar_source,
     )
@@ -91,17 +92,6 @@ class UpstreamAlreadyImported(BzrError):
         BzrError.__init__(self, version=str(version))
 
 
-class PreviousVersionTagMissing(BzrError):
-
-    _fmt = ("Unable to find the tag for the "
-            "previous upstream version (%(version)s) in the branch: "
-            "%(tag_name)s")
-
-    def __init__(self, version, tag_name):
-        super(PreviousVersionTagMissing, self).__init__(
-            version=version, tag_name=tag_name)
-
-
 class DscCache(object):
 
     def __init__(self, transport=None):
@@ -313,9 +303,9 @@ class DistributionBranch(object):
         :return: a String with the name of the tag.
         """
         if vendor is not None:
-            return '%s/%s' % (vendor, version)
+            return mangle_version(self.branch, '%s/%s' % (vendor, version))
         else:
-            return str(version)
+            return mangle_version(self.branch, str(version))
 
     def has_version(self, version, md5=None, vendor=None):
         """Whether this branch contains the package version specified.
@@ -876,17 +866,19 @@ class DistributionBranch(object):
                                  files_excluded=None):
         """Import an upstream part to the upstream branch.
 
-        :param tarballs: List of tarballs / components to extract
-        :param version: The upstream version to import.
-        :param parents: The tarball-branch parents to use for the import.
+        Args:
+          tarballs: List of tarballs / components to extract
+          version: The upstream version to import.
+          parents: The tarball-branch parents to use for the import.
             If an upstream branch is supplied, its automatically added to
             parents.
-        :param upstream_branch: An upstream branch to associate with the
+          upstream_branch: An upstream branch to associate with the
             tarball.
-        :param upstream_revisions: Upstream revision ids dictionary
-        :param md5sum: hex digest of the md5sum of the tarball, if known.
-        :return: list with (component, tag, revid, pristine_tar_imported)
-            tuples
+          upstream_revisions: Upstream revision ids dictionary
+          md5sum: hex digest of the md5sum of the tarball, if known.
+        Returns:
+          list with (component, tag, revid, pristine_tar_imported)
+          tuples
         """
         with _extract_tarballs_to_tempdir(tarballs) as tarball_dir:
             return self.import_upstream(
@@ -1366,10 +1358,12 @@ class DistributionBranch(object):
                     conflicts = self.tree.merge_from_branch(
                         self.pristine_upstream_branch,
                         merge_type=merge_type, from_revision=from_revision)
+                if not isinstance(conflicts, list):
+                    conflicts = self.tree.conflicts()
             else:
                 # Pull so that merge-upstream allows you to start a branch
                 # from upstream tarball.
-                conflicts = 0
+                conflicts = []
                 self.tree.pull(self.pristine_upstream_branch)
             self.pristine_upstream_branch.tags.merge_to(self.branch.tags)
             return conflicts, imported_revids
