@@ -18,11 +18,10 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 def load_tests(loader, basic_tests, pattern):
   testmod_names = [
           'test_builddeb',
+          'test_debrelease',
           'test_dep3',
           'test_do',
           'test_get_tar',
