@@ -19,8 +19,6 @@
 #    along with bzr-builddeb; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-from __future__ import absolute_import
-
 import string
 
 from debian.changelog import Version
@@ -85,10 +83,13 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubup.merge_from_branch(
             debp.branch, to_revision=self.revid_debp_n_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubup.conflicts()
+
         # There are two conflicts in the 'c' and the 'debian/changelog' files
         # respectively.
-        self.assertEquals(conflicts, 2)
-        conflict_paths = sorted([c.path for c in ubup.conflicts()])
+        self.assertEquals(len(conflicts), 2)
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'c.moved', u'debian/changelog'])
 
         # Undo the failed merge.
@@ -152,9 +153,11 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubup.merge_from_branch(
             debp.branch, to_revision=self.revid_debp_n_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubup.conflicts()
+
         # And, voila, only the packaging branch conflict remains.
-        self.assertEquals(conflicts, 1)
-        conflict_paths = sorted([c.path for c in ubup.conflicts()])
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'debian/changelog'])
 
     def test_deb_upstream_conflicts_with_ubu_packaging(self):
@@ -203,10 +206,12 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubup.merge_from_branch(
             debp.branch, to_revision=self.revid_debp_o_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubup.conflicts()
+
         # There are two conflicts in the 'c' and the 'debian/changelog' files
         # respectively.
-        self.assertEquals(conflicts, 2)
-        conflict_paths = sorted([c.path for c in ubup.conflicts()])
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'c.moved', u'debian/changelog'])
 
         # Undo the failed merge.
@@ -226,9 +231,11 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubup.merge_from_branch(
             debp.branch, to_revision=self.revid_debp_o_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubup.conflicts()
+
         # And, voila, only the packaging branch conflict remains.
-        self.assertEquals(conflicts, 1)
-        conflict_paths = sorted([c.path for c in ubup.conflicts()])
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'debian/changelog'])
 
     def test_upstreams_not_diverged(self):
@@ -247,9 +254,11 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubuntup.merge_from_branch(
             debianp.branch, to_revision=self.revid_debianp_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubuntup.conflicts()
+
         # There is only a conflict in the 'debian/changelog' file.
-        self.assertEquals(conflicts, 1)
-        conflict_paths = sorted([c.path for c in ubuntup.conflicts()])
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'debian/changelog'])
 
         # Undo the failed merge.
@@ -269,9 +278,11 @@ class MergePackageTests(TestCaseWithTransport):
         conflicts = ubuntup.merge_from_branch(
             debianp.branch, to_revision=self.revid_debianp_C)
 
+        if not isinstance(conflicts, list):
+            conflicts = ubuntup.conflicts()
+
         # The packaging branch conflict we saw above is still there.
-        self.assertEquals(conflicts, 1)
-        conflict_paths = sorted([c.path for c in ubuntup.conflicts()])
+        conflict_paths = sorted([c.path for c in conflicts])
         self.assertEquals(conflict_paths, [u'debian/changelog'])
 
     def _setup_debian_upstream_newer(self):
