@@ -22,8 +22,6 @@
 
 """Tests for the upstream module."""
 
-from __future__ import absolute_import
-
 from base64 import standard_b64encode
 
 import bz2
@@ -80,7 +78,8 @@ from ..upstream.branch import (
     UpstreamBranchSource,
     _upstream_branch_version,
     upstream_tag_to_version,
-    upstream_version_add_revision
+    upstream_version_add_revision,
+    PreviousVersionTagMissing,
     )
 from ..upstream.pristinetar import (
     get_pristine_tar_source,
@@ -319,10 +318,10 @@ class RecordingSource(UpstreamSource):
         self._recent = recent
 
     def get_latest_version(self, package, current_version):
-        return self._latest
+        return self._latest, self._latest
 
     def get_recent_versions(self, package, since_version=None):
-        return [v for v in self._recent
+        return [(v, v) for v in self._recent
                 if since_version is None or v > since_version]
 
     def fetch_tarballs(self, package, version, target_dir, components=None):
@@ -351,18 +350,22 @@ class StackedUpstreamSourceTests(TestCase):
         a = RecordingSource(False, latest="1.1")
         b = RecordingSource(False, latest="1.2")
         stack = StackedUpstreamSource([a, b])
-        self.assertEquals("1.1", stack.get_latest_version("mypkg", "1.0"))
+        self.assertEquals(("1.1", "1.1"), stack.get_latest_version("mypkg", "1.0"))
 
     def test_get_recent_versions(self):
         a = RecordingSource(False, recent=["1.0", "1.1"])
         b = RecordingSource(False, recent=["1.1", "1.2"])
         stack = StackedUpstreamSource([a, b])
         self.assertEquals(
-            ["1.0", "1.1", "1.2"], stack.get_recent_versions("mypkg", "0.9"))
+            [("1.0", "1.0"),
+             ("1.1", "1.1"),
+             ("1.2", "1.2"),
+             ], stack.get_recent_versions("mypkg", "0.9"))
         self.assertEquals(
-            ["1.0", "1.1", "1.2"], stack.get_recent_versions("mypkg"))
+            [("1.0", "1.0"), ("1.1", "1.1"), ("1.2", "1.2")],
+            stack.get_recent_versions("mypkg"))
         self.assertEquals(
-            ["1.2"], stack.get_recent_versions("mypkg", "1.1"))
+            [("1.2", "1.2")], stack.get_recent_versions("mypkg", "1.1"))
 
     def test_repr(self):
         self.assertEquals(
@@ -386,39 +389,34 @@ class GuessUpstreamRevspecTests(TestCase):
 
     def test_guess_upstream_revspec(self):
         self.assertEqual(
-                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2', u'tag:v.1.2'],
+                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2', u'tag:v.1.2', 'tag:release-1.2', 'tag:1_2_release', 'tag:1_2', 'tag:version-1.2', 'tag:foo-1_2-release', 'tag:foo-v1.2'],
                 list(guess_upstream_revspec('foo', '1.2')))
 
     def test_guess_upstream_revspec_ds(self):
         self.assertEqual(
-                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2', u'tag:v.1.2'],
+                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2', u'tag:v.1.2', 'tag:release-1.2', 'tag:1_2_release', 'tag:1_2', 'tag:version-1.2', 'tag:foo-1_2-release', 'tag:foo-v1.2'],
                 list(guess_upstream_revspec('foo', '1.2+ds')))
 
     def test_snapshot(self):
         self.assertEqual(
-                [u'revno:1200', u'tag:1.2+bzr1200', u'tag:foo-1.2+bzr1200',
-                 u'tag:v1.2+bzr1200', u'tag:v.1.2+bzr1200'],
+                [u'revno:1200', u'tag:1.2+bzr1200', u'tag:foo-1.2+bzr1200'],
                 list(guess_upstream_revspec('foo', '1.2+bzr1200')))
         self.assertEqual(
-                [u'revno:1200', u'tag:1.2~bzr1200', u'tag:foo-1.2~bzr1200',
-                 u'tag:v1.2~bzr1200', u'tag:v.1.2~bzr1200'],
+                [u'revno:1200', u'tag:1.2+bzr.1200', u'tag:foo-1.2+bzr.1200'],
+                list(guess_upstream_revspec('foo', '1.2+bzr.1200')))
+        self.assertEqual(
+                [u'revno:1200', u'tag:1.2~bzr1200', u'tag:foo-1.2~bzr1200'],
                 list(guess_upstream_revspec('foo', '1.2~bzr1200')))
         self.assertEqual(
                 [u'git:abc1def', u'date:2018-01-01', u'tag:1.2~git20180101.abc1def',
-                 u'tag:foo-1.2~git20180101.abc1def',
-                 u'tag:v1.2~git20180101.abc1def',
-                 u'tag:v.1.2~git20180101.abc1def'],
+                 u'tag:foo-1.2~git20180101.abc1def'],
                 list(guess_upstream_revspec('foo', '1.2~git20180101.abc1def')))
         self.assertEqual(
                 [u'git:abc1def', u'date:2018-01-01', u'tag:1.2~git20180101.0.abc1def',
-                 u'tag:foo-1.2~git20180101.0.abc1def',
-                 u'tag:v1.2~git20180101.0.abc1def',
-                 u'tag:v.1.2~git20180101.0.abc1def'],
+                 u'tag:foo-1.2~git20180101.0.abc1def'],
                 list(guess_upstream_revspec('foo', '1.2~git20180101.0.abc1def')))
         self.assertEqual(
-                [u'date:2018-01-01', u'tag:1.2~git20180101',
-                 u'tag:foo-1.2~git20180101',
-                 u'tag:v1.2~git20180101', u'tag:v.1.2~git20180101'],
+                [u'date:2018-01-01', u'tag:1.2~git20180101', u'tag:foo-1.2~git20180101'],
                 list(guess_upstream_revspec('foo', '1.2~git20180101')))
 
 
@@ -450,9 +448,9 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
         self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
         source = UpstreamBranchSource(self.tree.branch,
             {"2.1": self.tree.branch.last_revision().decode('utf-8')})
-        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+        self.assertEquals(("2.1", "2.1"), source.get_latest_version("foo", "2.1"))
         self.tree.commit("msg")
-        self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
+        self.assertEquals(("2.1+bzr2", "2.1+bzr2"), source.get_latest_version("foo", "2.1"))
 
     def test_get_latest_version_not_snapshot(self):
         revid1 = self.tree.commit("msg")
@@ -461,25 +459,42 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
         self.tree.branch.tags.set_tag("2.1", revid2)
         source = UpstreamBranchSource(self.tree.branch,
             {"2.1": self.tree.branch.last_revision().decode('utf-8')},
-            snapshot=False)
-        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+            version_kind="release")
+        self.assertEquals(("2.1", "2.1"), source.get_latest_version("foo", "1.0"))
         self.tree.commit("msg")
-        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+        self.assertEquals(("2.1", "2.1"), source.get_latest_version("foo", "1.0"))
+
+    def test_get_latest_version_irreversible_version(self):
+        self.tree.commit("msg")
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("0.01_01", revid2)
+        source = UpstreamBranchSource(self.tree.branch, {}, version_kind="release")
+        self.assertEquals(("0.01_01", "0.0101"), source.get_latest_version("foo", None))
 
     def test_get_recent_versions(self):
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("1.0", revid1)
         revid2 = self.tree.commit("msg")
-        self.tree.branch.tags.set_tag("2.1", revid2)
+        self.tree.branch.tags.set_tag("2.9", revid2)
+        source = UpstreamBranchSource(
+            self.tree.branch,
+            {"2.9": self.tree.branch.last_revision().decode('utf-8')})
+        self.assertEquals([("2.9", "2.9")], source.get_recent_versions("foo", "1.0"))
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag('2.10', revid2)
+        self.assertEquals(
+            [("2.9", "2.9"), ("2.10", "2.10")],
+            source.get_recent_versions("foo", "1.0"))
+
+    def test_get_recent_versions_previous_missing(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("1.0", revid1)
         source = UpstreamBranchSource(
             self.tree.branch,
             {"2.1": self.tree.branch.last_revision().decode('utf-8')})
-        self.assertEquals(["2.1"], source.get_recent_versions("foo", "1.0"))
-        revid2 = self.tree.commit("msg")
-        self.tree.branch.tags.set_tag('2.2', revid2)
-        self.assertEquals(
-            ["2.1", "2.2"],
-            source.get_recent_versions("foo", "1.0"))
+        self.assertRaises(
+            PreviousVersionTagMissing, source.get_recent_versions,
+            "foo", "1.1")
 
     def test_version_as_revisions(self):
         revid1 = self.tree.commit("msg")
@@ -547,13 +562,13 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
             self.tree.branch,
             {"2.1": self.tree.branch.last_revision().decode('utf-8')},
             local_dir=local_repo.controldir)
-        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+        self.assertEqual(('2.1', '2.1'), source.get_latest_version("foo", "2.1"))
         self.tree.commit("msg")
         source = UpstreamBranchSource.from_branch(
             self.tree.branch,
             {"2.1": self.tree.branch.last_revision().decode('utf-8')},
             local_dir=local_repo.controldir)
-        self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
+        self.assertEqual(("2.1+bzr2", "2.1+bzr2"), source.get_latest_version("foo", "2.1"))
 
 
 class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
@@ -589,9 +604,9 @@ class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
         source = LazyUpstreamBranchSource(self.tree.branch.base,
             {"2.1": self.tree.branch.last_revision()})
         self.assertIs(None, source._upstream_branch)
-        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+        self.assertEquals(("2.1", "2.1"), source.get_latest_version("foo", "1.0"))
         self.tree.commit("msg")
-        self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
+        self.assertEquals(("2.1+bzr2", "2.1+bzr2"), source.get_latest_version("foo", "1.0"))
         self.assertIsNot(None, source._upstream_branch)
 
     def test_get_recent_versions(self):
@@ -603,11 +618,11 @@ class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
             self.tree.branch.base,
             {"2.1": self.tree.branch.last_revision()})
         self.assertIs(None, source._upstream_branch)
-        self.assertEquals(["2.1"], source.get_recent_versions("foo", "1.0"))
+        self.assertEquals([("2.1", "2.1")], source.get_recent_versions("foo", "1.0"))
         revid3 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("2.2", revid3)
         self.assertEquals(
-            ["2.1", "2.2"],
+            [("2.1", "2.1"), ("2.2", "2.2")],
             source.get_recent_versions("foo", "1.0"))
         self.assertIsNot(None, source._upstream_branch)
 
@@ -632,44 +647,80 @@ class TestUpstreamBranchVersion(TestCase):
     """Test that the upstream version of a branch can be determined correctly.
     """
 
-    def get_suffix(self, version_string, revid):
+    def get_suffix(self, version_string, revid, sep):
         revno = len(self.revhistory) - self.revhistory.index(revid)
         if "bzr" in version_string:
             return "%sbzr%d" % (version_string.split("bzr")[0], revno)
-        return "%s+bzr%d" % (version_string, revno)
+        return "%s%sbzr%d" % (version_string, sep, revno)
 
     def test_snapshot_none_existing(self):
-      self.revhistory = [b"somerevid"]
-      self.assertEquals("1.2+bzr1",
-          _upstream_branch_version(self.revhistory, b"somerevid", {}, "bla", "1.2", self.get_suffix))
+        self.revhistory = [b"somerevid"]
+        self.assertEquals(
+            ("1.2+bzr1", "1.2+bzr1"),
+            _upstream_branch_version(
+                self.revhistory, b"somerevid", {}, "bla",
+                "1.2", self.get_suffix))
 
     def test_snapshot_nothing_new(self):
-      self.revhistory = []
-      self.assertEquals("1.2",
-          _upstream_branch_version(self.revhistory, NULL_REVISION, {}, "bla", "1.2", self.get_suffix))
+        self.revhistory = []
+        self.assertEquals(
+            ("1.2", "1.2"),
+            _upstream_branch_version(
+                self.revhistory, NULL_REVISION, {}, "bla", "1.2",
+                self.get_suffix))
 
     def test_new_tagged_release(self):
-      """Last revision is tagged - use as upstream version."""
-      self.revhistory = [b"somerevid"]
-      self.assertEquals("1.3",
-          _upstream_branch_version(self.revhistory, b"somerevid", {b"somerevid": [u"1.3"]}, "bla", "1.2", self.get_suffix))
+        """Last revision is tagged - use as upstream version."""
+        self.revhistory = [b"somerevid", b"oldrevid"]
+        self.assertEquals(
+            ("1.3", "1.3"),
+            _upstream_branch_version(
+                self.revhistory, b"somerevid",
+                {b"somerevid": [u"1.3", u"1.3a"], b"oldrevid": ["1.2"]}, "bla",
+                "1.2", self.get_suffix))
+
+    def test_snapshot_since_new_tagged_release(self):
+        """Last revision is tagged - use as upstream version."""
+        self.revhistory = [b"lastrevid", b"somerevid", b"oldrevid"]
+        self.assertEquals(
+            ("1.3+bzr3", "1.3+bzr3"),
+            _upstream_branch_version(
+                self.revhistory, b"lastrevid",
+                {b"somerevid": [u"1.3"], b'oldrevid': ["1.2"]}, "bla",
+                "1.2", self.get_suffix))
+
+    def test_snapshot_before_new_tagged_release(self):
+        """Last revision is tagged - use as upstream version."""
+        self.revhistory = [b"somerevid", b"targetrevid"]
+        self.assertEquals(
+            ("1.3~bzr1", "1.3~bzr1"),
+            _upstream_branch_version(
+                self.revhistory, b"targetrevid",
+                {b"somerevid": [u"1.3"]}, "bla", "1.2", self.get_suffix))
 
     def test_refresh_snapshot_pre(self):
-      self.revhistory = [b"somerevid", b"oldrevid"]
-      self.assertEquals("1.3~bzr2",
-          _upstream_branch_version(self.revhistory, b"somerevid", {}, "bla", "1.3~bzr1",
-              self.get_suffix))
+        self.revhistory = [b"somerevid", b"oldrevid"]
+        self.assertEquals(
+            ("1.3~bzr2", "1.3~bzr2"),
+            _upstream_branch_version(
+                self.revhistory, b"somerevid", {}, "bla",
+                "1.3~bzr1",
+                self.get_suffix))
 
     def test_refresh_snapshot_post(self):
-      self.revhistory = [b"somerevid", b"oldrevid"]
-      self.assertEquals("1.3+bzr2",
-          _upstream_branch_version(self.revhistory, b"somerevid", {}, "bla", "1.3+bzr1",
-              self.get_suffix))
+        self.revhistory = [b"somerevid", b"oldrevid"]
+        self.assertEquals(
+            ("1.3+bzr2", "1.3+bzr2"),
+            _upstream_branch_version(
+                self.revhistory, b"somerevid", {}, "bla", "1.3+bzr1",
+                self.get_suffix))
 
     def test_new_tag_refresh_snapshot(self):
-      self.revhistory = [b"newrevid", b"somerevid", b"oldrevid"]
-      self.assertEquals("1.3+bzr3",
-            _upstream_branch_version(self.revhistory, b"newrevid",
+        self.revhistory = [b"newrevid", b"somerevid", b"oldrevid"]
+        self.assertEquals(
+            ("1.3+bzr3", "1.3+bzr3"),
+            _upstream_branch_version(
+                self.revhistory, b"newrevid",
                 {b"somerevid": [u"1.3"]}, "bla", "1.2+bzr1", self.get_suffix))
 
 
@@ -687,6 +738,10 @@ class TestUpstreamTagToVersion(TestCase):
     def test_plain(self):
         self.assertEquals("2.0", upstream_tag_to_version(u"2.0"))
 
+    def test_v(self):
+        self.assertEquals("2.0", upstream_tag_to_version(u"v2.0"))
+        self.assertEquals("2.0", upstream_tag_to_version(u"v.2.0"))
+
     def test_package_prefix(self):
         self.assertEquals("42.0", upstream_tag_to_version(u"bla-42.0", "bla"))
 
@@ -696,12 +751,20 @@ class TestUpstreamTagToVersion(TestCase):
                 upstream_tag_to_version(u"bla-42.0\xa9", "bla"))
 
     def test_popt(self):
-        self.assertEquals("1.4", upstream_tag_to_version("popt-1_4", "popt"))
+        self.assertEquals("1_4", upstream_tag_to_version("popt-1_4", "popt"))
         self.assertEquals(
-            "1.4", upstream_tag_to_version("popt-1_4-release", "popt"))
+            "1_4", upstream_tag_to_version("popt-1_4-release", "popt"))
 
     def test_perl(self):
-        self.assertEquals("0.006019", upstream_tag_to_version("v0.006_019"))
+        self.assertEquals("0.006_019", upstream_tag_to_version("v0.006_019"))
+
+    def test_alpha(self):
+        self.assertEquals("1.4a1", upstream_tag_to_version("1.4a1", "popt"))
+        self.assertEquals("1.4a", upstream_tag_to_version("1.4a", "popt"))
+
+    def test_no_package(self):
+        self.assertEquals("1.4a1", upstream_tag_to_version("erbium-1.4a1", None))
+        self.assertEquals(None, upstream_tag_to_version("erbium-blah", None))
 
 
 class TestUpstreamVersionAddRevision(TestCaseWithTransport):
@@ -771,7 +834,7 @@ class TestUpstreamVersionAddRevision(TestCaseWithTransport):
 
     def test_git_new_rev(self):
         self.assertEquals(
-            "1.3+git20180101.e7f47cf",
+            "1.3+git20180101.1.e7f47cf",
             upstream_version_add_revision(
                 self, "1.3", b"somegitrev"))
         self.assertEquals(
@@ -954,7 +1017,7 @@ upstream-tag = blah-%(version%~%-)s
         self.source = GitPristineTarSource.from_tree(self.tree)
         self.assertEqual(
             self.source.tag_name("0.1~1"),
-            "blah-0.1-1")
+            "blah-0.1_1")
 
     def test_upstream_tag_name(self):
         upstream_v_no = "0.1"
@@ -970,7 +1033,7 @@ upstream-tag = blah-%(version%~%-)s
     def test_version(self):
         self.assertEquals(
             ['upstream/3.3', 'upstream-3.3', 'upstream_3.3', '3.3',
-             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3'],
+             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3', 'v/3.3', 'v.3.3'],
             self.source.possible_tag_names("pkg", "3.3", component=None))
 
     def test_version_with_tilde(self):
@@ -981,13 +1044,14 @@ upstream-tag = blah-%(version%~%-)s
             'upstream_3.3~brz232',
             '3.3~brz232',
             'v3.3~brz232',
-            'pkg-3.3~brz232'],
+            'pkg-3.3~brz232',
+            'v/3.3~brz232', 'v.3.3~brz232'],
             self.source.possible_tag_names(
                 'pkg', "3.3~brz232", component=None))
 
     def test_version_component(self):
         self.assertEquals([
-            'upstream-3.3/extlib'],
+            'upstream-3.3/extlib', 'upstream/3.3/extlib'],
             self.source.possible_tag_names('pkg', "3.3", component="extlib"))
 
     def test_version_as_revisions_missing(self):
@@ -1067,7 +1131,7 @@ class BzrPristineTarSourceTests(TestCaseWithTransport):
     def test_version(self):
         self.assertEquals(
             ['upstream/3.3', 'upstream-3.3', 'upstream_3.3', '3.3',
-             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3'],
+             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3', 'v/3.3', 'v.3.3'],
             self.source.possible_tag_names("pkg", "3.3", component=None))
 
     def test_version_with_tilde(self):
@@ -1078,13 +1142,14 @@ class BzrPristineTarSourceTests(TestCaseWithTransport):
             'upstream_3.3~brz232',
             '3.3~brz232',
             'v3.3~brz232',
-            'pkg-3.3~brz232'],
+            'pkg-3.3~brz232',
+            'v/3.3~brz232', 'v.3.3~brz232'],
             self.source.possible_tag_names(
                 'pkg', "3.3~brz232", component=None))
 
     def test_version_component(self):
         self.assertEquals([
-            'upstream-3.3/extlib'],
+            'upstream-3.3/extlib', 'upstream/3.3/extlib'],
             self.source.possible_tag_names('pkg', "3.3", component="extlib"))
 
     def test_version_as_revisions_missing(self):
@@ -1153,27 +1218,25 @@ class TarfileSourceTests(TestCaseWithTransport):
     def test_version(self):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
         self.assertEquals(
-            "1.0", source.get_latest_version("foo", "0.9"))
+            ("1.0", "1.0"), source.get_latest_version("foo", "0.9"))
 
     def test_version_unicode(self):
         source = TarfileSource(u"foo-1.0.tar.gz", u"1.0")
         latest_version = source.get_latest_version("foo", "0.9")
-        self.assertEquals("1.0", latest_version)
-        self.assertIsInstance(latest_version, str)
+        self.assertEquals(("1.0", "1.0"), latest_version)
 
     def test_version_unicode_not_specified(self):
         source = TarfileSource(u"foo-1.0.tar.gz")
         latest_version = source.get_latest_version("foo", u"0.9")
-        self.assertEquals("1.0", latest_version)
-        self.assertIsInstance(latest_version, str)
+        self.assertEquals(("1.0", "1.0"), latest_version)
 
     def test_get_latest_version_parses(self):
         source = TarfileSource("foo-1.0.tar.gz")
-        self.assertEquals("1.0", source.get_latest_version("foo", "0.9"))
+        self.assertEquals(("1.0", "1.0"), source.get_latest_version("foo", "0.9"))
 
     def test_get_recent_versions(self):
         source = TarfileSource("foo-1.0.tar.gz")
-        self.assertEquals(["1.0"], source.get_recent_versions("foo", "0.9"))
+        self.assertEquals([("1.0", "1.0")], source.get_recent_versions("foo", "0.9"))
 
     def test_fetch_tarballs(self):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
