@@ -18,8 +18,6 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 import bz2
 import gzip
 import hashlib
@@ -54,7 +52,6 @@ from ..util import (
     find_changelog,
     find_extra_authors,
     find_thanks,
-    get_build_architecture,
     get_files_excluded,
     get_commit_info_from_changelog,
     guess_build_type,
@@ -185,7 +182,10 @@ bzr-builddeb (0.16.2) unstable; urgency=low
     def test_find_changelog_lq_unversioned_debian_symlink(self):
         # LarstiQ mode, but with an unversioned "debian" -> "." symlink.
         # Bug 619295
-        self.requireFeature(SymlinkFeature)
+        try:
+            self.requireFeature(SymlinkFeature(self.test_dir))
+        except TypeError:  # brz < 3.2
+            self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
         tree.add(['changelog'])
@@ -212,7 +212,10 @@ bzr-builddeb (0.16.2) unstable; urgency=low
 
     def test_find_changelog_symlink(self):
         """When there was a symlink debian -> . then the code used to break"""
-        self.requireFeature(SymlinkFeature)
+        try:
+            self.requireFeature(SymlinkFeature(self.test_dir))
+        except TypeError:  # brz < 3.2
+            self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
         tree.add(['changelog'])
@@ -981,12 +984,6 @@ class TreeContainsUpstreamSourceTests(TestCaseWithTransport):
         self.assertFalse(tree_contains_upstream_source(tree))
 
 
-class BuildArchitectureTests(TestCase):
-
-    def test_is_str(self):
-        self.assertIsInstance(get_build_architecture(), text_type)
-
-
 class FilesExcludedTests(TestCaseWithTransport):
 
     def test_file_missing(self):
