@@ -18,13 +18,19 @@
 #    along with bzr-builddeb; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+from contextlib import ExitStack
 from debian.changelog import Version
 import os
 import re
 import subprocess
 import tempfile
+from typing import Optional, Tuple, Iterable
 
-from debmutate.versions import git_snapshot_data_from_version
+from debmutate.versions import (
+    git_snapshot_data_from_version,
+    get_snapshot_revision as _get_snapshot_revision,
+    debianize_upstream_version,
+    )
 
 from .... import osutils
 from ....branch import (
@@ -37,19 +43,18 @@ from ....errors import (
     InvalidRevisionId,
     NoSuchRevision,
     NoSuchTag,
+    NotBranchError,
     RevisionNotPresent,
     UnsupportedOperation,
     )
+from ....memorybranch import MemoryBranch
 from ..repack_tarball import get_filetype, repack_tarball
 from ....revision import NULL_REVISION
 from ....revisionspec import RevisionSpec
-from ....trace import note, mutter
+from ....trace import note, mutter, warning
 from ....tree import Tree
 
-try:
-    from ....revisionspec import InvalidRevisionSpec
-except ImportError:  # Breezy < 3.2
-    from ....errors import InvalidRevisionSpec
+from ....revisionspec import InvalidRevisionSpec
 
 from ..errors import (
     MultipleUpstreamTarballsNotSupported,
@@ -63,6 +68,20 @@ from . import (
 from ....export import (
     export,
     )
+from ....workingtree import (
+    WorkingTree,
+    )
+
+
+class PreviousVersionTagMissing(BzrError):
+
+    _fmt = ("Unable to find the tag for the "
+            "previous upstream version (%(version)s) in the upstream branch: "
+            "%(tag_name)s")
+
+    def __init__(self, version, tag_name):
+        super(PreviousVersionTagMissing, self).__init__(
+            version=version, tag_name=tag_name)
 
 
 def upstream_tag_to_version(tag_name, package=None):
@@ -71,19 +90,24 @@ def upstream_tag_to_version(tag_name, package=None):
         tag_name = tag_name[:-len('-release')]
     if tag_name.startswith("release-"):
         tag_name = tag_name[len("release-"):]
-    if tag_name[0] == "v" and tag_name[1].isdigit():
-        tag_name = tag_name[1:]
+    if tag_name.startswith('version-'):
+        tag_name = tag_name[len("version-"):]
     if (package is not None and (
           tag_name.startswith("%s-" % package) or
           tag_name.startswith("%s_" % package))):
         tag_name = tag_name[len(package)+1:]
-    if '_' in tag_name and '.' not in tag_name:
-        tag_name = tag_name.replace('_', '.')
-    if tag_name.count('_') == 1 and tag_name.startswith('0.'):
-        # This is a style commonly used for perl packages.
-        # Most debian packages seem to just drop the underscore.
-        tag_name = tag_name.replace('_', '')
-    if all([c.isdigit() or c in (".", "~") for c in tag_name]):
+    if package is None and '-' in tag_name:
+        (before, version) = tag_name.split('-', 1)
+        if before.isalpha() and not version[0].isalpha():
+            return version
+    if len(tag_name) >= 2 and tag_name[0] == "v" and tag_name[1].isdigit():
+        tag_name = tag_name[1:]
+    if len(tag_name) >= 3 and tag_name[0] == "v" and tag_name[1] in ('/', '.') and tag_name[2].isdigit():
+        tag_name = tag_name[2:]
+    if all([c.isdigit() or c in (".", "~", "_") for c in tag_name]):
+        return tag_name
+    parts = tag_name.split('.')
+    if len(parts) > 1 and all(p.isdigit() for p in parts[:-1]) and parts[-1].isalnum():
         return tag_name
     return None
 
@@ -107,11 +131,12 @@ def _upstream_branch_version(
     :param previous_version: Previous upstream version in debian changelog.
     :param add_rev: Function that can add a revision suffix to a version
         string.
-    :return: Name of the upstream revision.
+    :return: Tuple with vesion of the upstream revision, and mangled
     """
     if upstream_revision == NULL_REVISION:
         # No new version to merge
-        return previous_version
+        return previous_version, previous_version
+    last_upstream: Optional[Tuple[Version, str, str]] = None
     try:
         for r in revhistory:
             if r in reverse_tag_dict:
@@ -122,16 +147,38 @@ def _upstream_branch_version(
                     upstream_version = upstream_tag_to_version(
                             tag, package=package)
                     if upstream_version is not None:
-                        if r != upstream_revision:
-                            upstream_version = add_rev(
-                                str(upstream_version), upstream_revision)
-                        return upstream_version
+                        mangled_version = debianize_upstream_version(upstream_version, package)
+                        if r == upstream_revision:
+                            # Well, that's simple
+                            return upstream_version, mangled_version
+                        if last_upstream is None or Version(last_upstream[1]) < Version(mangled_version):
+                            last_upstream = (upstream_version, mangled_version, '+')
+            if r == upstream_revision and last_upstream:
+                # The last upstream release was after us
+                last_upstream = (last_upstream[0], last_upstream[1], '~')
     except RevisionNotPresent:
         # Ghost revision somewhere on mainline.
         pass
-    if previous_version is None:
-        return None
-    return add_rev(str(previous_version), upstream_revision)
+    if last_upstream is None:
+        # Well, we didn't find any releases
+        if previous_version is None:
+            last_upstream = ('0', '0', '+')
+        else:
+            # Assume we were just somewhere after the last release
+            last_upstream = (previous_version, previous_version, '+')
+    else:
+        if previous_version is not None and Version(last_upstream[1]) < Version(previous_version):
+            if '~' not in previous_version:
+                warning(
+                    'last found upstream version %s (%s) is lower than '
+                    'previous packaged upstream version (%s)',
+                    last_upstream[1], last_upstream[0], previous_version)
+                last_upstream = (previous_version, previous_version, '+')
+            else:
+                last_upstream = (previous_version, previous_version, '~')
+    upstream_version = add_rev(last_upstream[0], upstream_revision, last_upstream[2])
+    mangled_upstream_version = add_rev(last_upstream[1], upstream_revision, last_upstream[2])
+    return upstream_version, mangled_upstream_version
 
 
 def extract_gitid(rev):
@@ -176,6 +223,7 @@ def extract_svn_revno(rev):
             return svn_revno
 
 
+# TODO(jelmer): Rely on version from debmutate
 def upstream_version_add_revision(
         upstream_branch, version_string, revid, sep='+'):
     """Update the revision in a upstream version string.
@@ -231,7 +279,7 @@ def upstream_version_add_revision(
     if svn_revno:
         return "%s%ssvn%d" % (version_string, sep, svn_revno)
     elif gitid:
-        return "%s%sgit%s.%s" % (version_string, sep, gitdate, gitid)
+        return "%s%sgit%s.1.%s" % (version_string, sep, gitdate, gitid)
     else:
         return "%s%sbzr%s" % (version_string, sep, revno_str)
 
@@ -252,19 +300,20 @@ def get_snapshot_revision(upstream_version):
         upstream branch that the snapshot was taken from, or None if it
         doesn't appear to be a snapshot.
     """
-    match = re.search("(?:~|\\+)bzr([0-9]+)$", upstream_version)
-    if match is not None:
-        return match.groups()[0]
-    match = re.search("(?:~|\\+)svn([0-9]+)$", upstream_version)
-    if match is not None:
-        return "svn:%s" % match.groups()[0]
-    match = re.match(r"^(.*)([\+~])git(\d{8})\.([a-f0-9]{7})$",
-                     upstream_version)
-    if match:
-        return "git:%s" % match.group(4)
-    match = re.match(r"^(.*)([\+~])git(\d{8})$", upstream_version)
-    if match:
-        return "date:%s" % match.group(3)
+    ret = _get_snapshot_revision(upstream_version)
+    if ret is None:
+        return None
+    (kind, rev) = ret
+    if kind == 'svn':
+        return "svn:%s" % rev
+    elif kind == 'git':
+        return "git:%s" % rev
+    elif kind == 'date':
+        return 'date:%s' % rev
+    elif kind == 'bzr':
+        return str(rev)
+    else:
+        raise ValueError(kind)
     return None
 
 
@@ -302,8 +351,8 @@ def upstream_branch_version(upstream_branch, upstream_revision, package,
             revhistory, upstream_revision,
             upstream_branch.tags.get_reverse_tag_dict(), package,
             previous_version,
-            lambda version, revision: upstream_version_add_revision(
-                upstream_branch, version, revision))
+            lambda version, revision, sep: upstream_version_add_revision(
+                upstream_branch, version, revision, sep))
 
 
 def get_export_upstream_revision(config=None, version=None):
@@ -324,89 +373,48 @@ def get_export_upstream_revision(config=None, version=None):
     return rev
 
 
+def guess_upstream_tag(package, version, is_snapshot: bool = False) -> Iterable[str]:
+    yield version
+    if package:
+        for prefix in ['rust-']:
+            if package.startswith(prefix):
+                yield '%s-%s' % (package[len(prefix):], version)
+        yield '%s-%s' % (package, version)
+    if not is_snapshot:
+        yield 'v%s' % version
+        yield 'v.%s' % version
+        yield 'release-%s' % version
+        yield '%s_release' % version.replace('.', '_')
+        yield '%s' % version.replace('.', '_')
+        yield 'version-%s' % version
+        if package:
+            yield '%s-%s-release' % (package, version.replace('.', '_'))
+            yield '%s-v%s' % (package, version)
+
+
 def guess_upstream_revspec(package, version):
     """Guess revspecs matching an upstream version string."""
     if version.endswith('+ds'):
         version = str(version)[:-len('+ds')]
+    if version.endswith('+dfsg'):
+        version = str(version)[:-len('+dfsg')]
+    if version.endswith('+repack'):
+        version = str(version)[:-len('+repack')]
+    is_snapshot = False
     if "+bzr" in version or "~bzr" in version:
-        yield "revno:%s" % re.match(".*[~+]bzr(\\d+).*", version).group(1)
+        is_snapshot = True
+        m = re.match(r".*[~+]bzr(\.?)(\d+).*", version)
+        if m:
+            yield "revno:%s" % m.group(2)
     (git_id, git_date) = git_snapshot_data_from_version(version)
     if git_id:
+        is_snapshot = True
         yield "git:%s" % git_id
     if git_date:
+        is_snapshot = True
         yield "date:%s" % git_date
-    yield 'tag:%s' % version
-    for prefix in ['rust-']:
-        if package.startswith(prefix):
-            yield 'tag:%s-%s' % (package[len(prefix):], version)
-    yield 'tag:%s-%s' % (package, version)
-    yield 'tag:v%s' % version
-    yield 'tag:v.%s' % version
-
-
-try:
-    from breezy.memorybranch import MemoryBranch
-except ImportError:  # breezy < 3.1.1
-    from ....bzr import branch as bzr_branch
-    from ....lock import _RelockDebugMixin, LogicalLockResult
-
-    class MemoryBranch(bzr_branch.Branch, _RelockDebugMixin):
-
-        def __init__(self, repository, last_revision_info, tags):
-            from ....tag import DisabledTags, MemoryTags
-            self.repository = repository
-            self._last_revision_info = last_revision_info
-            self._revision_history_cache = None
-            if tags is not None:
-                self.tags = MemoryTags(tags)
-            else:
-                self.tags = DisabledTags(self)
-            self._partial_revision_history_cache = []
-            self._last_revision_info_cache = None
-            self._revision_id_to_revno_cache = None
-            self._partial_revision_id_to_revno_cache = {}
-            self._partial_revision_history_cache = []
-
-        def lock_read(self):
-            self.repository.lock_read()
-            return LogicalLockResult(self.unlock)
-
-        def lock_write(self, token=None):
-            self.repository.lock_write()
-            return BranchWriteLockResult(self.unlock, None)
-
-        def unlock(self):
-            self.repository.unlock()
-
-        def last_revision_info(self):
-            return self._last_revision_info
-
-        def _gen_revision_history(self):
-            """Generate the revision history from last revision
-            """
-            last_revno, last_revision = self.last_revision_info()
-            self._extend_partial_history()
-            return list(reversed(self._partial_revision_history_cache))
-
-        def get_rev_id(self, revno, history=None):
-            """Find the revision id of the specified revno."""
-            with self.lock_read():
-                if revno == 0:
-                    return NULL_REVISION
-                last_revno, last_revid = self.last_revision_info()
-                if revno == last_revno:
-                    return last_revid
-                if last_revno is None:
-                    self._extend_partial_history()
-                    return self._partial_revision_history_cache[
-                            len(self._partial_revision_history_cache) - revno]
-                else:
-                    if revno <= 0 or revno > last_revno:
-                        raise NoSuchRevision(self, revno)
-                    distance_from_last = last_revno - revno
-                    if len(self._partial_revision_history_cache) <= distance_from_last:
-                        self._extend_partial_history(distance_from_last)
-                    return self._partial_revision_history_cache[distance_from_last]
+    for tag in guess_upstream_tag(package, version, is_snapshot):
+        yield 'tag:%s' % tag
 
 
 class UpstreamBranchSource(UpstreamSource):
@@ -418,12 +426,12 @@ class UpstreamBranchSource(UpstreamSource):
 
     def __init__(self, upstream_branch, upstream_revision_map=None,
                  config=None, actual_branch=None, create_dist=None,
-                 other_repository=None, snapshot=True):
+                 other_repository=None, version_kind="auto"):
         self.upstream_branch = upstream_branch
         self._actual_branch = actual_branch or upstream_branch
         self.create_dist = create_dist
         self.config = config
-        self.snapshot = snapshot
+        self.version_kind = version_kind
         self.other_repository = other_repository
         self.upstream_revision_map = {}
         if upstream_revision_map is not None:
@@ -432,7 +440,7 @@ class UpstreamBranchSource(UpstreamSource):
     @classmethod
     def from_branch(cls, upstream_branch, upstream_revision_map=None,
                     config=None, local_dir=None, create_dist=None,
-                    snapshot=True):
+                    version_kind="auto"):
         """Create a new upstream branch source from a branch.
 
         This will optionally fetch into a local directory.
@@ -456,7 +464,7 @@ class UpstreamBranchSource(UpstreamSource):
             upstream_branch=upstream_branch,
             upstream_revision_map=upstream_revision_map, config=config,
             actual_branch=actual_branch, create_dist=create_dist,
-            snapshot=snapshot)
+            version_kind=version_kind)
 
     def version_as_revision(self, package, version, tarballs=None):
         if version in self.upstream_revision_map:
@@ -483,6 +491,10 @@ class UpstreamBranchSource(UpstreamSource):
                 raise PackageVersionNotPresent(package, version, self)
         raise PackageVersionNotPresent(package, version, self)
 
+    def revision_tree(self, package, version):
+        revid = self.version_as_revision(package, version)
+        return self.upstream_branch.repository.revision_tree(revid)
+
     def version_as_revisions(self, package, version, tarballs=None):
         # FIXME: Support multiple upstream locations if there are multiple
         # components
@@ -498,52 +510,78 @@ class UpstreamBranchSource(UpstreamSource):
         else:
             return True
 
+    def get_latest_snapshot_version(self, package, current_version):
+        revid = self.upstream_branch.last_revision()
+        version, mangled_version = self.get_version(package, current_version, revid)
+        if mangled_version is not None:
+            self.upstream_revision_map[mangled_version] = 'revid:%s' % revid.decode('utf-8')
+        return version, mangled_version
+
+    def get_latest_release_version(self, package, current_version):
+        versions = list(self.get_recent_versions(package, current_version))
+        if not versions:
+            return None
+        return versions[-1]
+
     def get_latest_version(self, package, current_version):
-        if self.snapshot:
-            revid = self.upstream_branch.last_revision()
-            version = self.get_version(package, current_version, revid)
-            self.upstream_revision_map[version] = 'revid:%s' % revid.decode('utf-8')
+        if self.version_kind == "snapshot":
+            return self.get_latest_snapshot_version(package, current_version)
+        elif self.version_kind == "release":
+            version = self.get_latest_release_version(package, current_version)
+            if version is None:
+                # TODO(jelmer): De-debianize current_version ?
+                return current_version, current_version
+            return version
+        elif self.version_kind == "auto":
+            version = self.get_latest_release_version(package, current_version)
+            if version is None:
+                note(gettext('No upstream releases found, falling back to snapshot.'))
+                version = self.get_latest_snapshot_version(package, current_version)
             return version
         else:
-            versions = list(self.get_recent_versions(package, current_version))
-            if not versions:
-                return None
-            return versions[-1]
+            raise ValueError(self.version_kind)
 
-    def get_recent_versions(self, package, since_version=None):
+    def get_recent_versions(
+            self, package: str, since_version: Optional[Version] = None):
         versions = []
         tags = self.upstream_branch.tags.get_tag_dict()
         with self.upstream_branch.repository.lock_read():
             graph = self.upstream_branch.repository.get_graph()
             if since_version is not None:
-                since_revision = self.version_as_revision(
-                    package, since_version)
+                try:
+                    since_revision = self.version_as_revision(
+                        package, since_version)
+                except PackageVersionNotPresent:
+                    raise PreviousVersionTagMissing(package, since_version)
             else:
                 since_revision = None
             for tag, revision in tags.items():
                 version = upstream_tag_to_version(tag, package)
                 if version is None:
                     continue
-                if since_version is not None and version <= since_version:
+                mangled_version = debianize_upstream_version(version, package)
+                self.upstream_revision_map[mangled_version] = 'tag:%s' % tag
+                if since_version is not None and mangled_version <= since_version:
                     continue
                 if since_revision and not graph.is_ancestor(
                         since_revision, revision):
                     continue
-                versions.append(version)
-        return sorted(versions)
+                versions.append((version, mangled_version))
+        return sorted(versions, key=lambda v: Version(v[1]))
 
     def get_version(self, package, current_version, revision):
         with self.upstream_branch.lock_read():
             return upstream_branch_version(
                 self.upstream_branch, revision, package, current_version)
 
-    def fetch_tarballs(self, package, version, target_dir, components=None,
-                       revisions=None):
+    def fetch_tarballs(
+            self, package: str, version,
+            target_dir, components=None, revisions=None):
         if components is not None and components != [None]:
             # Multiple components are not supported
             raise PackageVersionNotPresent(package, version, self)
-        note("Looking for upstream %s/%s in upstream branch %r.",
-             package, version, self.upstream_branch)
+        note("Looking for upstream %s in upstream branch %r.",
+             version, getattr(self, '_actual_branch', self.upstream_branch))
         with self.upstream_branch.lock_read():
             if revisions is not None:
                 revid = revisions[None]
@@ -591,9 +629,9 @@ class LazyUpstreamBranchSource(UpstreamBranchSource):
 
     def __init__(self, upstream_branch_url, upstream_revision_map=None,
                  config=None, create_dist=None, other_repository=None,
-                 snapshot=True):
+                 version_kind="snapshot"):
         self.upstream_branch_url = upstream_branch_url
-        self.snapshot = snapshot
+        self.version_kind = version_kind
         self._upstream_branch = None
         self.config = config
         self.create_dist = create_dist
@@ -620,28 +658,82 @@ class DistCommandFailed(BzrError):
 
     _fmt = "Dist command failed to produce a tarball: %(error)s"
 
-    def __init__(self, error):
-        super(DistCommandFailed, self).__init__(error=error)
+    def __init__(self, error, kind=None):
+        super(DistCommandFailed, self).__init__(error=error, kind=kind)
+
+
+def _dupe_vcs_tree(tree, directory):
+    with tree.lock_read():
+        if isinstance(tree, WorkingTree):
+            tree = tree.basis_tree()
+    result = tree._repository.controldir.sprout(
+        directory, create_tree_if_local=True,
+        revision_id=tree.get_revision_id()
+    )
+    if not result.has_workingtree():
+        raise AssertionError
+    # Copy parent location - some scripts need this
+    if isinstance(tree, WorkingTree):
+        parent = tree.branch.get_parent()
+    else:
+        try:
+            parent = tree._repository.controldir.open_branch().get_parent()
+        except NotBranchError:
+            parent = None
+    if parent:
+        result.open_branch().set_parent(parent)
 
 
 def run_dist_command(
-        rev_tree: Tree, package: str, version: Version, target_dir: str,
-        dist_command: str) -> bool:
-    with tempfile.TemporaryDirectory() as td:
-        package_dir = os.path.join(td, package)
-        export(rev_tree, package_dir, 'dir')
-        existing_files = os.listdir(package_dir)
-        env = dict(os.environ.items())
-        env['PACKAGE'] = package
-        env['VERSION'] = version
-        note('Running dist command: %s', dist_command)
+        rev_tree: Tree, package: Optional[str], version: Version, target_dir: str,
+        dist_command: str, include_controldir: bool = False) -> bool:
+
+    def _run_and_interpret(command, env, dir):
         try:
-            subprocess.check_call(
-                dist_command, env=env, cwd=package_dir, shell=True)
+            subprocess.check_call(command, env=env, cwd=dir, shell=True)
         except subprocess.CalledProcessError as e:
             if e.returncode == 2:
-                return None
-            raise DistCommandFailed(str(e))
+                raise NotImplementedError
+            try:
+                import json
+                with open(env['DIST_RESULT'], 'r') as f:
+                    result = json.load(f)
+                raise DistCommandFailed(
+                    result['description'], result['result_code'])
+            except FileNotFoundError:
+                raise DistCommandFailed(str(e))
+
+    with ExitStack() as es:
+        td = es.enter_context(tempfile.TemporaryDirectory())
+        if package:
+            package_dir = os.path.join(td, package)
+        else:
+            package_dir = os.path.join(td, 'package')
+        env = dict(os.environ.items())
+        if package:
+            env['PACKAGE'] = package
+        env['VERSION'] = version
+        env['DIST_RESULT'] = os.path.join(td, 'dist.json')
+        note('Running dist command: %s', dist_command)
+        if include_controldir:
+            _dupe_vcs_tree(rev_tree, package_dir)
+        else:
+            export(rev_tree, package_dir, 'dir')
+        existing_files = os.listdir(package_dir)
+        try:
+            _run_and_interpret(dist_command, env, package_dir)
+        except NotImplementedError:
+            return None
+        except DistCommandFailed as e:
+            # Retry with the control directory
+            if (e.kind == 'vcs-control-directory-needed' and
+                    not include_controldir):
+                osutils.rmtree(package_dir)
+                _dupe_vcs_tree(rev_tree, package_dir)
+                existing_files = os.listdir(package_dir)
+                _run_and_interpret(dist_command, env, package_dir)
+            else:
+                raise
         new_files = os.listdir(package_dir)
         diff_files = set(new_files) - set(existing_files)
         diff = [n for n in diff_files if get_filetype(n) is not None]
@@ -658,7 +750,7 @@ def run_dist_command(
                     os.rename(entry.path, os.path.join(target_dir, entry.name))
                     return entry.name
             note('No tarballs found in dist directory.')
-        diff = set(os.listdir(td)) - set([package])
+        diff = set(os.listdir(td)) - set([os.path.basename(package_dir)])
         if len(diff) == 1:
             fn = diff.pop()
             note('Found tarball %s in parent directory.', fn)
