@@ -18,6 +18,7 @@
 #    along with bzr-builddeb; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+from typing import Optional
 
 from base64 import (
     standard_b64decode,
@@ -71,15 +72,13 @@ from .branch import (
     RevisionSpec,
     )
 from .tags import (
-    GbpTagFormatError,
-    gbp_expand_tag_name,
-    upstream_tag_name,
     is_upstream_tag,
     possible_upstream_tag_names,
     search_for_upstream_version,
     upstream_version_tag_start_revids,
     upstream_tag_version,
     )
+from debmutate.vcs import gbp_expand_tag_name
 from debmutate.versions import mangle_version_for_git
 
 
@@ -95,7 +94,7 @@ class PristineTarDeltaTooLarge(PristineTarError):
 
 
 class PristineTarDeltaAbsent(PristineTarError):
-    _fmt = 'There is not delta present for %(version)s.'
+    _fmt = 'There is no delta present for %(version)s.'
 
     def __init__(self, version):
         BzrError.__init__(self, version=version)
@@ -286,6 +285,9 @@ def revision_pristine_tar_format(rev):
 
 class BasePristineTarSource(UpstreamSource):
 
+    def _has_revision(self, revid, md5):
+        raise NotImplementedError(self._has_revision)
+
     def tag_version(self, version, revid, component=None):
         """Tags the upstream branch's last revision with an upstream version.
 
@@ -325,7 +327,8 @@ class BasePristineTarSource(UpstreamSource):
                 package, version, component, md5)
         return ret
 
-    def has_version(self, package, version, tarballs=None):
+    def has_version(
+            self, package: Optional[str], version: Version, tarballs=None):
         if tarballs is None:
             return self.has_version_component(package, version, component=None)
         else:
@@ -344,8 +347,9 @@ class BasePristineTarSource(UpstreamSource):
             ret.setdefault(version, {})[component] = tag_revid
         return ret
 
-    def version_component_as_revision(self, package, version, component,
-                                      md5=None):
+    def version_component_as_revision(
+            self, package: Optional[str], version: Version,
+            component: Optional[str], md5: Optional[str] = None):
         with self.branch.lock_read():
             for tag_name in self.possible_tag_names(
                     package, version, component=component):
@@ -361,7 +365,8 @@ class BasePristineTarSource(UpstreamSource):
             (git_id, git_date) = git_snapshot_data_from_version(version)
             if git_id:
                 try:
-                    return RevisionSpec.from_string('git:%s' % git_id).as_revision_id(self.branch)
+                    revspec = RevisionSpec.from_string('git:%s' % git_id)
+                    return revspec.as_revision_id(self.branch)
                 except (InvalidRevisionSpec, NoSuchTag):
                     pass
             revid = self._search_for_upstream_version(
@@ -382,8 +387,11 @@ class BasePristineTarSource(UpstreamSource):
             self, package, version, component, md5=None):
         raise NotImplementedError(self._search_for_upstream_version)
 
-    def has_version_component(self, package, version, component, md5=None):
-        for tag_name in self.possible_tag_names(package, version, component=component):
+    def has_version_component(
+            self, package: Optional[str], version: Version, component,
+            md5=None):
+        for tag_name in self.possible_tag_names(
+                package, version, component=component):
             try:
                 revid = self.branch.tags.lookup_tag(tag_name)
             except NoSuchTag:
@@ -412,7 +420,13 @@ class BzrPristineTarSource(BasePristineTarSource):
         :param distro: Optional distribution name
         :return: a String with the name of the tag.
         """
-        return upstream_tag_name(version, component, distro, False)
+        if distro is None:
+            name = "upstream-" + version
+        else:
+            name = "upstream-%s-%s" % (distro, version)
+        if component is not None:
+            name += "/%s" % component
+        return name
 
     def import_component_tarball(
             self, package, version, tree, parent_ids,
@@ -460,8 +474,6 @@ class BzrPristineTarSource(BasePristineTarSource):
         if md5 is not None:
             if supports_custom_revprops:
                 revprops["deb-md5"] = md5
-            else:
-                message += ", md5 %s" % md5
             delta = make_pristine_tar_delta_from_tree(
                 tree, tarball, subdir=subdir, exclude=exclude)
             if supports_custom_revprops:
@@ -533,7 +545,9 @@ class BzrPristineTarSource(BasePristineTarSource):
             raise PackageVersionNotPresent(package, version, self)
         return target_filename
 
-    def possible_tag_names(self, package, version, component):
+    def possible_tag_names(
+            self, package: Optional[str], version: Version,
+            component: Optional[str]):
         return possible_upstream_tag_names(package, version, component)
 
     def get_pristine_tar_delta(self, package, version, dest_filename,
@@ -713,8 +727,15 @@ class GitPristineTarSource(BasePristineTarSource):
         :return: a String with the name of the tag.
         """
         if self.gbp_tag_format is not None:
-            return gbp_expand_tag_name(self.gbp_tag_format, version)
-        return upstream_tag_name(version, component, distro, True)
+            return gbp_expand_tag_name(self.gbp_tag_format, mangle_version_for_git(version))
+        # In git, the convention is to use a slash
+        if distro is None:
+            name = "upstream/" + mangle_version_for_git(version)
+        else:
+            name = "upstream-%s/%s" % (distro, mangle_version_for_git(version))
+        if component is not None:
+            name += "/%s" % component
+        return name
 
     def import_component_tarball(
             self, package, version, tree, parent_ids,
@@ -755,8 +776,6 @@ class GitPristineTarSource(BasePristineTarSource):
         revprops = {}
         if component is not None:
             message += ", component %s" % component
-        if md5 is not None:
-            message += ", md5 %s" % md5
         if author is not None:
             revprops['authors'] = author
         timezone = None
@@ -848,16 +867,24 @@ class GitPristineTarSource(BasePristineTarSource):
         return target_filename
 
     def _has_revision(self, revid, md5=None):
+        if not self.branch.repository.has_revision(revid):
+            return False
+        if self.branch.last_revision() == _mod_revision.NULL_REVISION:
+            # there's no explicit upstream branch so.. whatever?
+            return True
         with self.branch.lock_read():
             graph = self.branch.repository.get_graph()
             if not graph.is_ancestor(revid, self.branch.last_revision()):
+                warning(
+                    'Revision %r exists in repository but not in '
+                    'upstream branch, ignoring', revid)
                 return False
         return True
 
-    def possible_tag_names(self, package, version, component):
+    def possible_tag_names(self, package: Optional[str], version: Version, component: Optional[str]):
         tags = []
         if self.gbp_tag_format:
-            tags.append(gbp_expand_tag_name(self.gbp_tag_format, version))
+            tags.append(gbp_expand_tag_name(self.gbp_tag_format, mangle_version_for_git(version)))
 
         tags.extend(possible_upstream_tag_names(package, version, component))
 
@@ -967,7 +994,9 @@ class GitPristineTarSource(BasePristineTarSource):
                 package, version, component, target_dir)
             for component in components]
 
-    def _search_for_upstream_version(self, package, version, component, md5=None):
+    def _search_for_upstream_version(
+            self, package: Optional[str], version: Version, component,
+            md5=None):
         start_revids = []
         sources = []
         if self.branch.last_revision() != NULL_REVISION:
