@@ -18,8 +18,6 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from __future__ import absolute_import
-
 import errno
 import hashlib
 import signal
@@ -40,6 +38,7 @@ from debmutate.changelog import (
     find_last_distribution,
     strip_changelog_message,
     )
+from debmutate.versions import get_snapshot_revision
 
 from ... import (
     bugtracker,
@@ -56,6 +55,7 @@ from ...transport import (
     do_catching_redirections,
     get_transport,
     )
+from ...tree import Tree
 from . import (
     global_conf,
     )
@@ -170,7 +170,7 @@ class AddChangelogError(BzrError):
         BzrError.__init__(self, changelog=changelog)
 
 
-def find_changelog(t, subpath='', merge=False, max_blocks=1):
+def find_changelog(t, subpath='', merge=False, max_blocks=1, strict=False):
     """Find the changelog in the given tree.
 
     First looks for 'debian/changelog'. If "merge" is true will also
@@ -228,7 +228,8 @@ def find_changelog(t, subpath='', merge=False, max_blocks=1):
         contents = t.get_file_text(changelog_file)
     changelog = Changelog()
     changelog.parse_changelog(
-        contents, max_blocks=max_blocks, allow_empty_author=True)
+        contents, max_blocks=max_blocks, allow_empty_author=True,
+        strict=strict)
     return changelog, top_level
 
 
@@ -823,29 +824,15 @@ def dput_changes(path):
     subprocess.check_call(["dput", changes_file], cwd=bd)
 
 
-def debsign(path, keyid=None):
-    (bd, changes_file) = os.path.split(path)
-    args = ["debsign"]
-    if keyid:
-        args.append("-k%s" % keyid)
-    args.append(changes_file)
-    subprocess.check_call(args, cwd=bd)
-
-
-def changes_filename(package, version, arch):
+def find_changes_files(path, package, version):
     non_epoch_version = version.upstream_version
     if version.debian_version is not None:
         non_epoch_version += "-%s" % version.debian_version
-    return "%s_%s_%s.changes" % (package, non_epoch_version, arch)
-
-
-def get_build_architecture():
-    try:
-        return subprocess.check_output(
-            ['dpkg-architecture', '-qDEB_BUILD_ARCH']).strip().decode()
-    except subprocess.CalledProcessError as e:
-        raise BzrError(
-            "Could not find the build architecture: %s" % e)
+    c = re.compile('%s_%s_(.*).changes' % (re.escape(package), re.escape(non_epoch_version)))
+    for entry in os.scandir(path):
+        m = c.match(entry.name)
+        if m:
+            yield m.group(1), entry
 
 
 def get_files_excluded(tree, subpath='', top_level=False):
@@ -862,3 +849,38 @@ def get_files_excluded(tree, subpath='', top_level=False):
             return copyright.header["Files-Excluded"].split()
         except KeyError:
             return []
+
+
+def control_files_in_root(tree: Tree, subpath: str) -> bool:
+    debian_path = os.path.join(subpath, "debian")
+    if tree.has_filename(debian_path):
+        return False
+    control_path = os.path.join(subpath, "control")
+    if tree.has_filename(control_path):
+        return True
+    if tree.has_filename(control_path + ".in"):
+        return True
+    return False
+
+
+def full_branch_url(branch):
+    """Get the full URL for a branch.
+
+    Ideally this should just return Branch.user_url,
+    but that currently exclude the branch name
+    in some situations.
+    """
+    if branch.name is None:
+        return branch.user_url
+    url, params = urlutils.split_segment_parameters(branch.user_url)
+    if branch.name != "":
+        params["branch"] = urlutils.quote(branch.name, "")
+    return urlutils.join_segment_parameters(url, params)
+
+
+def detect_version_kind(upstream_version):
+    """Detect the version kind from the upstream version."""
+    snapshot_info = get_snapshot_revision(upstream_version)
+    if snapshot_info is None:
+        return 'release'
+    return 'snapshot'
